////////////////////////////////////////////////////////////////////////////////
// MinGW-w64 direct X 7 - ddraw test
////////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <tchar.h>
#include <D2D1.h>
#include <dwrite.h>

////////////////////////////////////////////////////////////////////////////////

ID2D1Factory*           pD2DFactory = NULL;
#if 0
ID2D1DCRenderTarget*    pD2DTarget   = NULL;
#else
ID2D1HwndRenderTarget*  pD2DTarget   = NULL;
#endif
ID2D1SolidColorBrush*   pBlackBrush = NULL;
IDWriteFactory*         pWFactory   = NULL;
IDWriteTextFormat*      pTextFormat = NULL;

////////////////////////////////////////////////////////////////////////////////

static int msx      = 0;
static int msy      = 0;
static int mslb     = 0; // Left Button.
static int msrb     = 0; // Right Button.

const int refWidth  = 1280;
const int refHeight = 720;

HWND g_hwnd;


////////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK DlgMsgLoop(HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam);

bool CreateMainDlg(int w,int h,TCHAR *ClassName,void* DlgMsgLoop,void* hInst);
void loppMain();
void ErrMsgOut(HWND hwnd,const TCHAR *str);

void DDDestroy();
void DDFlipBack(bool vw);

void PutDDPixel(int x,int y,unsigned short c);
void ClearDDPrime(unsigned long Color);

////////////////////////////////////////////////////////////////////////////////

bool DDCreate(HWND hwnd,int w,int h)
{
    HRESULT hr = E_FAIL;

    hr = ::D2D1CreateFactory( D2D1_FACTORY_TYPE_SINGLE_THREADED, &pD2DFactory );

    if ( hr == S_OK )
    {
        HDC dxDC = GetDC(g_hwnd);

        if ( dxDC == NULL )
        {
            return false;
        }

        SetLayout( dxDC, LAYOUT_BITMAPORIENTATIONPRESERVED );

        D2D1_RENDER_TARGET_PROPERTIES dxProp;

        // When it used forr D2D1_ALPHA_MODE_PREMULTIPLIED ?

#if 0
        dxProp = D2D1::RenderTargetProperties( D2D1_RENDER_TARGET_TYPE_DEFAULT,
                                               D2D1::PixelFormat(DXGI_FORMAT_B8G8R8A8_UNORM,  D2D1_ALPHA_MODE_IGNORE),
                                               0,
                                               0,
                                               D2D1_RENDER_TARGET_USAGE_NONE,
                                               D2D1_FEATURE_LEVEL_DEFAULT);

        hr = pD2DFactory->CreateDCRenderTarget( &dxProp, &pD2DTarget );
#else
        dxProp = D2D1::RenderTargetProperties();
        dxProp.usage = D2D1_RENDER_TARGET_USAGE_GDI_COMPATIBLE;

        D2D1_SIZE_U size = D2D1::SizeU( refWidth, refHeight );

        hr = pD2DFactory->CreateHwndRenderTarget( dxProp,
                                                  D2D1::HwndRenderTargetProperties( g_hwnd, size ),
                                                  &pD2DTarget );
#endif


        if ( hr == S_OK )
        {
#if 0
            RECT rc;

            GetClientRect( g_hwnd, &rc );

            pD2DTarget->BindDC( dxDC, &rc );
#endif
            pD2DTarget->SetTextAntialiasMode( D2D1_TEXT_ANTIALIAS_MODE_GRAYSCALE );
            pD2DTarget->CreateSolidColorBrush( D2D1::ColorF(D2D1::ColorF::White), &pBlackBrush );

        }

        DWriteCreateFactory( DWRITE_FACTORY_TYPE_SHARED,
                             __uuidof(IDWriteFactory),
                            reinterpret_cast<IUnknown**>(&pWFactory) );

        pWFactory->CreateTextFormat( TEXT("Tahoma"),
                                       NULL,
                                       DWRITE_FONT_WEIGHT_REGULAR,
                                       DWRITE_FONT_STYLE_NORMAL,
                                       DWRITE_FONT_STRETCH_NORMAL,
                                       12.0f,
                                       TEXT("en-us"),
                                       &pTextFormat );



        if ( hr == S_OK )
            return true;
    }

    return false;
}

void DDDestroy()
{

}

void DDFlipBack(bool vw)
{

}

void PutDDPixel(int x,int y,unsigned short c)
{

}

void ClearDDPrime(unsigned long Color)
{
    if ( pD2DTarget != NULL )
    {
        pD2DTarget->Clear( D2D1::ColorF( D2D1::ColorF::Black ) );
    }
}


void DrawGDIText(int x,int y,TCHAR *str,int r,int g,int b)
{
    if ( pD2DTarget != NULL )
    {
        IDWriteTextLayout*      textLayout  = NULL;
        ID2D1SolidColorBrush*   color       = NULL;


        pD2DTarget->CreateSolidColorBrush( D2D1::ColorF( r, b,b ), &color );

        pD2DTarget->DrawText( str,
                             _tcslen( str ),
                             pTextFormat,
                             D2D1::RectF(0,0, refWidth, refHeight),
                             color );

        delete color;
    }
}

//////////////////////////////////////////////////////////////

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;

    CreateMainDlg( refWidth, refHeight, TEXT("D2D Test"), (void*)DlgMsgLoop, hInstance );

    if(!g_hwnd)
        return FALSE;

    bool ret = DDCreate( g_hwnd, refWidth , refHeight );
    if(ret==FALSE)
        return FALSE;

    while(1)
    {
        if(PeekMessage(&msg,NULL,0,0,PM_NOREMOVE))
        {
            if(!GetMessage(&msg,NULL,0,0)) return msg.wParam;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            loppMain();
            WaitMessage();
        }
    }

    DDDestroy();
    ShowCursor(TRUE);
    return 1;
}

void ErrMsgOut(HWND hwnd, const TCHAR *str)
{
    MessageBox(hwnd,str,TEXT("Error."),MB_OK);
    DestroyWindow(hwnd);
}

bool CreateMainDlg(int w,int h,TCHAR *ClassName,void* DlgMsgLoop,void* hInst)
{
    WNDCLASS wc;

    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc      = (long (__stdcall *)(struct HWND__ *,unsigned int,unsigned int,long))DlgMsgLoop;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = (HINSTANCE)hInst;
    wc.hIcon            = LoadIcon((HINSTANCE)hInst,(const TCHAR *)IDI_APPLICATION);
    wc.hCursor          = LoadCursor(NULL,IDC_ARROW);
    wc.hbrBackground    = NULL;
    wc.lpszClassName    = ClassName;
    wc.lpszMenuName     = NULL;

    RegisterClass(&wc);

    g_hwnd = CreateWindowEx( 0,
                             ClassName,
                             ClassName,
                             WS_POPUP,
                             0,
                             0,
                             GetSystemMetrics(SM_CXSCREEN),
                             GetSystemMetrics(SM_CYSCREEN),
                             NULL,
                             NULL,
                             (struct HINSTANCE__ *)hInst,
                             NULL );

    if( g_hwnd == NULL )
        return FALSE;

    ShowWindow(g_hwnd,1);
    UpdateWindow(g_hwnd);
    SetFocus(g_hwnd);
    ShowCursor(FALSE);

    return TRUE;
}


/////////////////////////////////////////////////////////////////

LRESULT CALLBACK DlgMsgLoop(HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam)
{
    switch(message)
    {
        case WM_MOUSEMOVE:
            msx=LOWORD(lParam);
            msy=HIWORD(lParam);
            break;

        case WM_LBUTTONDOWN:
            mslb=1;
            break;

        case WM_RBUTTONDOWN:
            msrb=1;
            break;

        case WM_LBUTTONUP:
            mslb=0;
            break;

        case WM_RBUTTONUP:
            msrb=0;
            break;

        case WM_KEYDOWN:
        case WM_DESTROY:
            ShowCursor(TRUE);
            PostQuitMessage(0);
            break;
    }

    return DefWindowProc(hwnd,message,wParam,lParam);
}


void loppMain()
{
    if ( pD2DTarget != NULL )
    {
        pD2DTarget->BeginDraw();

        ClearDDPrime( 0 );

        pD2DTarget->SetTransform( D2D1::Matrix3x2F::Identity() );
        pD2DTarget->DrawEllipse( D2D1::Ellipse( D2D1::Point2F( float( msx ), float( msy ) ),
                                                 100.0f,
                                                 100.0f),
                                                 pBlackBrush,
                                                 3.0f );
        DrawGDIText( 10,10, TEXT("Testing ...."), 255,255,255 );

        pD2DTarget->EndDraw();
    }


    PutDDPixel(msx,msy, 0xFFFFFFFF);
    DDFlipBack(0);
}
